package member

import (
	"context"
	"fmt"
	"time"

	"encoding/json"

	"git.loafle.net/overflow/overflow_server_app/commons"
	"git.loafle.net/overflow/overflow_server_app/config"
	"git.loafle.net/overflow/overflow_server_app/external/grpc"
	jwt "github.com/dgrijalva/jwt-go"
	"github.com/valyala/fasthttp"
)

func SignIn(ctx *fasthttp.RequestCtx) {
	var err error
	var webParams map[string]interface{}

	webBytes := ctx.PostBody()
	err = json.Unmarshal(webBytes, &webParams)

	if err != nil {
		commons.SendRESTError(ctx, fasthttp.StatusBadRequest, err)
		// fmt.Fprintf(ctx, "Err!!!!: %s\n", err)
		return
	}

	length := len(webParams)

	if length < 0 {
		fmt.Println("eeee")
	}

	signinId := webParams["signinId"].(string)
	signinPw := webParams["signinPw"].(string)

	params := []string{signinId, signinPw}

	gRPCCtx := context.Background()
	r, err := grpc.Exec(gRPCCtx, "MemberService.signin", params)
	if nil != err {
		commons.SendRESTError(ctx, fasthttp.StatusBadRequest, err)
		return
	}

	token := jwt.New(jwt.SigningMethodRS256)

	/* Create a map to store our claims */
	claims := token.Claims.(jwt.MapClaims)

	/* Set token claims */
	claims["iss"] = "overFlow"
	claims["iat"] = time.Now().Unix()
	claims["exp"] = time.Now().Add(time.Hour * 24).Unix()
	claims["aud"] = "www.overflow.cloud"
	claims["sub"] = signinId

	/* Sign the token with our secret */
	tokenString, _ := token.SignedString([]byte(config.Config.Auth.SigningKey))

	var jwtCookie fasthttp.Cookie
	jwtCookie.SetKey("AuthToken")
	jwtCookie.SetValue(tokenString)
	ctx.Response.Header.SetCookie(&jwtCookie)

	commons.SendRESTResponse(ctx, r)
}
